import hashlib
import json
import time
import urllib
from base64 import b64encode, b64decode

AREA_CN = 'cn'  # 国内
AREA_GLOBAL = 'global'  # 海外

# 登录校验的地址
URL_CHECKUSERINFO = {
    # 国内请求
    AREA_CN: "https://usdk.ultrasdk.com/hu/v1/login/checkUserInfo.lg",
    # 海外请求
    AREA_GLOBAL: "https://uglobal.ultrasdk.com/hu/v1/login/checkUserInfo.lg",
}


class USDK(object):
    def __init__(self, PRODUCT_ID, PRODUCT_KEY, CALLBACK_KEY, SRV_AREA = AREA_GLOBAL, _logger=print):
        """

        :param PRODUCT_ID: 应用ID，如 D10000
        :param PRODUCT_KEY: 应用KEY
        :param CALLBACK_KEY: 支付回调的KEY，如果不接入支付则不使用
        """
        if not SRV_AREA in URL_CHECKUSERINFO:
            raise Exception("无效地区", SRV_AREA)
        self.URL_CHECKUSERINFO = URL_CHECKUSERINFO[SRV_AREA]
        self.PRODUCT_ID = PRODUCT_ID
        self.PRODUCT_KEY = PRODUCT_KEY
        self.CALLBACK_KEY = CALLBACK_KEY
        self._logger = _logger

    @classmethod
    def md5hex(cls, s):
        news = str(s).encode()  # 先变成bytes类型才能加密
        m = hashlib.md5(news)  # 创建md5对象
        return m.hexdigest()  # 获取加密后的字符串

    @classmethod
    def encode(cls, body):
        if isinstance(body, dict):
            body = json.dumps(body)
        data = b64encode(str(body).encode()).decode()
        if len(data) > 51:
            swap_index = [1, 33, 10, 42, 18, 50, 19, 51]  # 编码:置换规则
            ss = list(data)
            for i in range(1, len(swap_index), 2):
                l, r = swap_index[i - 1], swap_index[i]
                ss[l], ss[r] = ss[r], ss[l]
            return ''.join(ss)
        return data

    @classmethod
    def sign(cls, data, pcode, key, timestamp):
        msg = 'data={}&pcode={}&timestamp={}&{}'.format(data, pcode, timestamp, key)
        return cls.md5hex(msg)

    def login(self, cUid, cName, accessToken):
        """
        登录校验，如果返回对象无效，则表示验证客户端结果失败，请「拒绝登录」

        :param cUid: 渠道登录返回的用户id，「必须」
        :param cName: 渠道登录返回的username，「可选」
        :param accessToken: usdk登录返回的token，「必须」
        :return: JSON对象 {code:状态,msg:描述,cUid,cName,channel,channelId,swLogin}
        """
        if self._logger is not None:
            self._logger('登录信息 = cUid:{} cName:{} accessToken:{}'.format(cUid, cName, accessToken))

        data = self.encode({"accessToken": accessToken, "cName": cName, "cUid": cUid})
        headers = {'User-Agent': 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)'}
        values = {'data': data, 'pcode': self.PRODUCT_ID, 'timestamp': int(time.time()), }
        values['sign'] = self.sign(data, values['pcode'], self.PRODUCT_KEY, values['timestamp'])
        # 将字典格式化成能用的形式
        data = urllib.parse.urlencode(values).encode('utf-8')
        # 创建一个request,放入我们的地址、数据、头
        request = urllib.request.Request(self.URL_CHECKUSERINFO, data, headers)
        # 访问
        resp = urllib.request.urlopen(request).read().decode('utf-8')
        # 利用json解析包解析返回的json数据 拿到翻译结果
        if isinstance(resp, str):
            return json.loads(resp)

        if self._logger is not None:
            self._logger('登录验证，可能异常的返回:{}'.format(resp))

        return resp

    def pay_callback(self, data, sign):
        """
        校验 支付通知结果

        :param data:
        :param sign:
        :return: 'FAIL'或'SUCCESS',结果,附加信息
        """
        # print('回调内容：{}'.format(callback))
        # url解码，根据实际的项目框架来，看是否需要url解码
        if data is not None and '%' in data: data = urllib.parse.unquote(data)
        if sign is not None and '%' in sign: sign = urllib.parse.unquote(sign)

        if data is None or sign is None:
            return 'FAIL', "无效参数", sign

        # 支付业务的签名计算
        sign_data = self.md5hex('data={}&{}'.format(data, self.CALLBACK_KEY))

        if sign_data != sign:
            if self._logger is not None:
                self._logger('支付校验签名不一致，原签名={}，计算签名={}'.format(sign, sign_data))
            return 'FAIL', "签名验证失败", sign_data

        resp = b64decode(data.encode()).decode()
        if '{' == resp[0] and '}' == resp[-1]:
            resp = json.loads(resp)
            return 'SUCCESS', resp, sign_data
        return 'FAIL', '未知内容', resp


class AuthInfo(object):
    """
    登录校验结果
    """

    @classmethod
    def parse(self, r):
        if isinstance(r, dict):
            return AuthInfo(**r)
        return None

    def __init__(self, code: int, msg: str = None
                 , cUid: str = None, cName: str = None
                 , channelId: int = None, channel: str = None
                 , swLogin: int = None
                 , heroSdkUserId: str = None, ext=None
                 , **kwargs):
        self.code, self.msg = code, msg
        self.cUid, self.cName = cUid, cName
        self.channelId, self.channel = channelId, channel
        self.swLogin = swLogin
        self.heroSdkUserId = heroSdkUserId
        self.ext = ext
        for k, v in kwargs.items():
            setattr(self, k, v)

    def __str__(self):
        if self.code == 0:
            return "渠道的用户id,cUid={} " \
                   "渠道的账号,cName={} " \
                   "渠道ID,channelId={} " \
                   "渠道,channel={} " \
                   "切登录,swLogin={} ".format(
                self.cUid, self.cName, self.channelId, self.channel, self.swLogin
            )
        return "校验失败,code={} msg='{}'".format(self.code, self.msg)


class PayResult(object):
    """
    支付通知的业务数据
    """

    @classmethod
    def parse(self, r):
        if isinstance(r, dict):
            return PayResult(**r)
        return None

    def __init__(self, amount, gameOrder: str, orderNo: str, status: int, payTime: str, goodsId: str
                 , goodsName: str = None, channelUid=None, channelId=None, channel=None
                 , selfDefine: str = None
                 , dealAmount: str = None, sandbox: str = None, paytype: str = None
                 , iap_sub: str = None, iap_sub_expire: str = None
                 , yx_is_in_intro_offer_period: str = None
                 , yx_sub_type: str = None
                 , yx_is_trial_period: str = None
                 , **kwargs):
        """
        支付结果

        :param amount: Double	必须	SDK	成交金额，单位元
        :param gameOrder: String	必须	游戏	游戏订单号
        :param orderNo: String	必须	SDK	整合sdk订单号
        :param status: Integer	必须	SDK	交易状态，0表示成功
        :param payTime: String	必须	SDK	交易时间yyyy-MM-dd HH:mm:ss
        :param goodsId: String 必须 SDK	玩家购买的商品ID
        :param goodsName: String 可选 SDK	后台配置的商品名称
        :param channelUid: String	必须	SDK	渠道的用户id
        :param channelId: Integer   可选  SDK 渠道ID
        :param channel: String	必须	SDK	渠道类型
        :param selfDefine: String	可选	游戏	透传参数
        :param kwargs:
        """
        self.amount, self.gameOrder, self.orderNo = amount, gameOrder, orderNo
        self.status, self.payTime, self.goodsId, self.goodsName = status, payTime, goodsId, goodsName
        self.channelUid, self.channelId, self.channel = channelUid, channelId, channel
        self.selfDefine = selfDefine
        self.dealAmount, self.sandbox, self.paytype = dealAmount, sandbox, paytype
        self.iap_sub, self.iap_sub_expire = iap_sub, iap_sub_expire
        self.yx_is_in_intro_offer_period = yx_is_in_intro_offer_period
        self.yx_sub_type = yx_sub_type
        self.yx_is_trial_period = yx_is_trial_period
        for k, v in kwargs.items():
            setattr(self, k, v)

    def __str__(self):
        return "付款金额,amount={} " \
               "游戏订单号,gameOrder={} " \
               "USDK的订单号,orderNo={} " \
               "交易状态,status={} " \
               "支付时间,payTime={} " \
               "内购商品ID,goodsId={} " \
               "内购商品名称,goodsName={} " \
               "渠道用户ID,channelUid={} " \
               "渠道ID,channelId={} " \
               "渠道码,channel={} " \
               "自定义信息,selfDefine={} " \
               "".format(
            self.amount, self.gameOrder, self.orderNo, self.status, self.payTime,
            self.goodsId, self.goodsName,
            self.channelUid, self.channel, self.channelId, self.selfDefine
        )
